/*
   Multiple GA_FIT: multiple structure alignment


   Multiple sequence alignment written by Mark S.Johnson:
   Johnson et al. (1993) J.Mol.Biol. 231:735-752.

   Amino acid scoring matrices:
   Johnson and Overington (1993) J.Mol.Biol. 233:716-738.

   Applied to multiple structure superposition by Alex C.W.May:
   May and Johnson (1994) Prot.Eng. 7(4):475-485.

   Needs: at least 2 PDB structures



   Last revision:	13 August 1994

*/


#include <stdio.h>
#include <math.h>
#include "malign.h"
#include "mat10.h"

#define MAX_NAME 25
#define MAX_CDE 25                      /* max length code */
#define MAX_TIT  50                     /* max length title */
#define DEF_MAT   0			/* default matrix: birkbeck*/
#define DEF_PEN	 40			/* default non-negative gap penalty */

char pdb_filename[MAX_SEQ][MAX_NAME];	/* array of PDB filenames */
char code[MAX_SEQ][MAX_CDE];		/* seq ids */
char title[MAX_SEQ][MAX_TIT];		/* seq titles */
char seq[MAX_SEQ][MAX_LEN];		/* sequences */
float largest_dim[MAX_SEQ];
int matrix[MAX_LEN][MAX_LEN];	/* score matrix to be traced */
int len[MAX_SEQ];			/* length of seqs; may include gaps! */
int ren[MAX_SEQ];			/* real length of seqs */
int nel_p[MAX_SEQ][MAX_SEQ];		/* aligned positions include overhgs*/
int nscore[MAX_SEQ][MAX_SEQ];	/* align score */
int s_flag;				/* segment smoothing if TRUE */
int seg;				/* segment length to smooth over */
int temp[2][MAX_LEN];		/* temp storage for two seqs */
int family[MAX_SEQ][MAX_SEQ];	/* treed members */
float fscore[MAX_SEQ][MAX_SEQ];		/* norm align score */
float rscore[MAX_SEQ][MAX_SEQ];		/* raw align score */
float fpid[MAX_SEQ][MAX_SEQ];		/* seq ids */
float fdif[MAX_SEQ][MAX_SEQ];		/* seq diffs */

int p_f;		/* internal flag to control average over template*/
int m_f;		/* internal flag to average or not over template*/
int r_f;		/* full alignment on; trace out if 1; 0 = off */

int mf_flag=0;		/* places signal "^" about final align */

int ind[26]={ 0,21, 1, 2, 3, 4, 5, 6, 7,21, 8, 9,10,  /* 21 will be  */
             11,21,12,13,14,15,16,21,17,18,21,19,21}; /* out of bounds */


int read_um();
void comp_um();
int pair_um();
float norm_um();
void print_um();
void dist_um();
void tree_um();
float pidf_um();
float pids_um();
float pdif_um();
int score_um();


int main(int argc, char *argv[])
{

FILE *file, *infp;

register int i, j;

int ns;			/* number of sequences */
int pen;		/* pen value to use */
int mat;		/* mat number to use */
int score;		/* alignemnt score */
int t_len;		/* length of alignment */
int list1[MAX_SEQ];	/* list of seqs to align as set */
int list2[MAX_SEQ];	/* list of seqs to align as set */
int nlist1,nlist2;	/* number in set to be aligned with another set */
int lenj,lenk;		/* temp length of two sets of seqs */
int p_ct=0;		/* controls the working dots printout */
int set;		/* counts input args */
float tscore;		/* sum of normalized scores */
char m_names[MAX_MAT][19];	/* matrix names */
char Temp[20];		/* used to read args */

double alt_score;
int end_no_equiv;

int ignore_data0;       /* dummy variable for initdata(): need only cut_off */
double ignore_data1;    /* dummy variable for initdata(): need only cut_off */
double cut_off;
double sqr_cutoff;

int pair_no_equiv; /* pairwise equivalences */
double pair_rmsd;  /* pairwise rmsd */
double pfte;       /* pairwise fractional topological equivalence */

int pairwise_equivs[MAX_SEQ][MAX_SEQ]; /* pairwise equivalences in multipe superposition */
double pairwise_rmsd[MAX_SEQ][MAX_SEQ];/* pairwise rmsds in multipe superposition */
double sdist[MAX_SEQ][MAX_SEQ];        /* pairwise structural distance metric */

int *all_equiv; /* common equivalences */
register int common_no_equiv;
int **common_mat;
double common_pairwise_rmsd[MAX_SEQ][MAX_SEQ];/* pairwise rmsds in multipe superposition
						 over common (all pairwise) equivalences */



strcpy(m_names[0],"STRMAT97");
strcpy(m_names[1],"DAYHOFF");
strcpy(m_names[2],"DOOLITTLE");
strcpy(m_names[3],"FITCH");
strcpy(m_names[4],"GRANTHAM");
strcpy(m_names[5],"LEVIN");
strcpy(m_names[6],"MCLACHLAN");
strcpy(m_names[7],"MIYATA");
strcpy(m_names[8],"RAO");
strcpy(m_names[9],"RISLER");
strcpy(m_names[10],"GONNET");
strcpy(m_names[11],"HENIKOFF");
strcpy(m_names[12],"JONES");
strcpy(m_names[13],"PAM120");
strcpy(m_names[14],"BIRKBECK65");

/*
 *	open sequences file for reading
 */

	
if((file=fopen(argv[1],"r"))==NULL)
	{
	fprintf(stderr,"\n\n%s: Cannot open %s! (FATAL)",argv[0],argv[1]);
	fprintf(stderr,"\nUsage: %s list_of_PDB_files [ -seg=# ] [ -M# ] ",argv[0]);
	fprintf(stderr,"[ -P# ] [ -O ] [ > output ]");
	fprintf(stderr,"\n\nList_of_PDB_files -- structures in PDB format!");
	fprintf(stderr,"\n\tMaximum number of 3D structures: %d",MAX_SEQ);
	fprintf(stderr,"\n\tMaximum length of 3D structures: %d\n",MAX_LEN-52);
	fprintf(stderr,"\n-seg=#  : smooth over segment # residues long; try 6");
	fprintf(stderr,"\n-M  : scoring matrix");
	fprintf(stderr,"; -P  : gap penalty");
	fprintf(stderr,"; -O  : for binary alignment output\n");
	fprintf(stderr,"\nMatrix vals:   STRMAT97= 0  pen = 40; (DEFAULT)\n\n");
	for(i=1;i<MAX_MAT;i++)
		{
		if(i%2!=0)fprintf(stderr,"%-20.20s	= %d",m_names[i],i);
		else fprintf(stderr,"		%-20.20s	= %d\n",m_names[i],i);
		}
	fprintf(stderr,"\n%s\n",VER); 
	exit(1);
	}
else 	{
	printf("\n\n**************************************");
	printf("*************************************\n\n");
	printf("\n\n\t\t\tMULTIPLE GA_FIT\n\n\n");
	printf("\t\tAlex C.W.May and Mark S.Johnson\n\n");
	printf("\t\t\t    1994\n");
	}

/*
 *	enter matrix number; penalty and r_f (=1 for all pairwise aligns).
 */

mat=DEF_MAT;
pen=DEF_PEN;
s_flag=FALSE;

r_f=0;

set=2;

while(set<argc)
	{
	if(strncmp(argv[set],"-seg=",5)==0)
		{
		i=5;
		while(argv[set][i]!='\0')
			{
			Temp[i-5]=argv[set][i];
			i++;
			}
		Temp[i-5]='\0';
		seg=atoi(Temp);
		s_flag=TRUE;
		set++;
		}
	else if(strncmp(argv[set],"-M",2)==0)
		{
		i=2;
		while(argv[set][i]!='\0')
			{
			Temp[i-2]=argv[set][i];
			i++;
			}
		Temp[i-2]='\0';
		mat=atoi(Temp);
		set++;
		}
	else if(strncmp(argv[set],"-P",2)==0)
		{
		i=2;
		while(argv[set][i]!='\0')
			{
			Temp[i-2]=argv[set][i];
			i++;
			}
		Temp[i-2]='\0';
		pen=atoi(Temp);
		set++;
		}
	else if(strncmp(argv[set],"-O",2)==0)
		{
		r_f=1;
		set++;
		}
	else	{
		printf("\nUnrecogized option: %s \n",argv[set]);
		exit(49);
		}
	}

if(s_flag==TRUE)printf("\n\nSmoothing over segments %d long",seg);
else printf("\n");
printf("\nMatrix:  %s",m_names[mat]);
printf("    Penalty: %d",pen);

printf("\n\n**************************************");
printf("*************************************\n\n");

if(mat>=MAX_MAT)
	{
	fprintf(stderr,"\n\nMatrix value %d exceeds listings (FATAL)\n\n",mat);
	fprintf(stderr,"\nUsage: %s list_of_PDB_files [ -seg=# ] [ -M# ] ",argv[0]);
	fprintf(stderr,"[ -P# ] [ -O ] [ > output ]");
	fprintf(stderr,"\n\nList_of_PDB_files -- structures in PDB format!");
	fprintf(stderr,"\n\tMaximum number of 3D structures: %d",MAX_SEQ);
	fprintf(stderr,"\n\tMaximum length of 3D structures: %d\n",MAX_LEN-52);
	fprintf(stderr,"\n-seg=#  : smooth over segment # residues long; try 6");
	fprintf(stderr,"\n-M  : scoring matrix");
	fprintf(stderr,"; -P  : gap penalty");
	fprintf(stderr,"; -O  : for binary alignment output\n");
	fprintf(stderr,"\nMatrix vals:   STRMAT97= 0  pen = 40; (DEFAULT)\n\n");
	for(i=1;i<MAX_MAT;i++)
		{
		if(i%2!=0)fprintf(stderr,"%-20.20s	= %d",m_names[i],i);
		else fprintf(stderr,"		%-20.20s	= %d\n",m_names[i],i);
		}
	fprintf(stderr,"\n%s\n",VER); 
	exit(3);
	}

/*
 *	read the sequences of the 3-D structures into seq[0 ... n]
 */

ns=read_um(file);

/*
 *	only a pair so turn on printout
 */

if(ns==2)
	{
	r_f=1;
	mf_flag=1;	/* print ^ to signify final alignment */
	}

/*
 *	make pairwise alignments
 */

p_f=0;
m_f=0;

if(r_f==0)printf("\n\nWorking\n ");

for(i=0;i<ns-1;i++)
	{
	for(j=i+1;j<ns;j++)
		{

		/*
		 * comp_um produces matrix for pair_um to align from.
		 * m_f = 0 here; do not average over more than one
		 *            sequence.
		 */

		comp_um(i,j,mat,&lenj,&lenk);

		/*
 		 * pair_um aligns from the values given in matrix
		 * r_f=1 here; means trace out alignment
		 * and return score
		 */

		score=pair_um(lenj,lenk,pen,&t_len,1);

		/*
		 *	score separately; score from pair_um may be off
		 *	by a multiple of the gap penalty if the sequences
		 *	had gaps introduced into the sequences prior to
		 *	their comparison by malign. 1 will indicate
		 *	data from pair_um is located in temp[2][len]
		 *	rather than seq[][].
		 */

		score=score_um(i,j,t_len,mat,pen,1);

		nscore[i][j]=score;
		nscore[j][i]=score;

		/*
		 * norm_um normalizes the score by dividing by
		 * the average of the two self-self comparisons
		 * returns fscore
		 */


		fscore[i][j]=norm_um(i,j,t_len,score,mat,1);
		fscore[j][i]=fscore[i][j];

		if(r_f==1)printf("\n_______________________________");
		if(r_f==1)
		printf("___________________________________________\n\n");
		if(r_f==1)printf("Similarity score= %6d ",score);
		if(r_f==1)printf("Normalized score= %8.1f\n",fscore[i][j]);
		if(r_f==0)
			{
			if(p_ct>0 && (p_ct+1)%38 ==0)printf("\n");
			printf(". ");
			fflush(stdout);
			p_ct++;
			}


		list1[0]=i;nlist1=1;
		list2[0]=j;nlist2=1;


		/*
		 *	percentage identities -- pairwise only
		 */
		
		fpid[i][j]=pids_um(i,j,t_len);
		fpid[j][i]=fpid[i][j];

		}
	}

if(r_f==0)printf("\nDone\n");

fflush(stdout);

/*
 *	print the fids
 */

printf("\n\n******************************************************");
printf("*********************");
printf("\n************************ Pairwise comparison statistics:");
printf(" ******************");
printf("\n********************************************************");
printf("*******************");

printf("\n\nPercentage identities:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[i]);
	for(j=0;j<ns;j++)
		{
		if(i==j)fpid[i][j]=100.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.1f",fpid[i][j]);
		}
	}

printf("\n");

printf("\n\nNormalized alignment scores (NAS = 100.0 * Sreal/Smax):\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[i]);
	for(j=0;j<ns;j++)
		{
		if(i==j)fscore[i][j]=100.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.1f",fscore[i][j]);
		}
	}

printf("\n");

/* Pairwise structural comparison and exit here if ns==2 */

if(ns==2){

  printf("\n\n");

  /*
   *	make the alignment
   */

  score = ga_fit(list1,nlist1,list2,nlist2,&alt_score,&end_no_equiv,mat,pen,&t_len);

  /*
   *	printout the alignment
   */

  printf("\n_______________________________");
  printf("___________________________________________\n\n");
  printf("Cluster: [%s %s]",code[list1[0]],code[list2[0]]);
  printf("\nSimilarity score after Genetic algorithm: %d",(int)alt_score);
  printf("\nSimilarity score after Kearsley (1989) algorithm: %d\n",score);

  print_um(list1,nlist1,list2,nlist2,t_len,end_no_equiv);

  /*	Write CA atom fitted coordinate files */

  if((write_um(ns))!=0){
    fprintf(stderr,"Error in writing CA atom fitted coordinate files\n");
    exit(1);
  }

  exit(0);

}


/*
 *	create distances from norm align score -- pairwise only or scores
 */

dist_um(ns);

/*
 *	sort by hierarchy tree and multiply align as tree is constructed
 */

p_f=0;
r_f=1;
tree_um(ns,mat,pen,&t_len);

/*	Write CA atom fitted coordinate files */

if((write_um(ns))!=0){
  fprintf(stderr,"Error in writing CA atom fitted coordinate files\n");
  exit(1);
}

/*
 *	calculate percentage id for all pairs; then flush
 */

for(i=0;i<ns-1;i++)
	{
	for(j=i+1;j<ns;j++)
		{
		fpid[i][j]=pidf_um(i,j,t_len);
		fpid[j][i]=fpid[i][j];
		}
	}


/*
 *	calculate percentage difference for all pairs; then flush
 */

for(i=0;i<ns-1;i++)
	{
	for(j=i+1;j<ns;j++)
		{
		fdif[i][j]=pdif_um(i,j,t_len);
		fdif[j][i]=fdif[i][j];
		}
	}

/*
 *	calculate norm align score for all pairs; then flush
 */

tscore=0.0;
for(i=0;i<ns;i++)
	{
	for(j=i;j<ns;j++)
		{
		score=score_um(i,j,t_len,mat,pen,0);
		rscore[i][j]= (float)score;
		rscore[j][i]= (float)score;
		if(i!=j)tscore+= rscore[i][j];
		fscore[i][j]=norm_um(i,j,t_len,score,mat,0);
		fscore[j][i]=fscore[i][j];
		}
	}

/* Need cut-off distance for equivalences to calculate pairwise
   structural distance metric according to Johnson et al. (1990)
   J.Mol.Evol. 30: 43-59.
*/

/* Open ga_fit.inp file */

if((infp=fopen(INPUT,"r"))==NULL){
  fprintf(stderr,"File %s cannot be opened\n",INPUT);
  exit(1);
}

if((initdata(infp,&ignore_data0,&cut_off,&ignore_data1))!=0){
   fprintf(stderr,"Error in reading file %s\n",INPUT);
   exit(1);
}

/* Close ga_fit.inp file */

fclose(infp);

sqr_cutoff = SQR(cut_off);

all_equiv=ivector(0,t_len-1);

for(i=0;i<t_len;i++)
   all_equiv[i]=0;

/*
   Scan each pairwise permutation of molecules to define common
   equivalences: Sutcliffe et al.(1987) Protein Eng. 1(5): 377-384.
*/

for(i=0;i<ns-1;i++){

   for(j=i+1;j<ns;j++){

      if((pairwise_comp(i,j,sqr_cutoff,&pair_no_equiv,&pair_rmsd,&pfte,
			all_equiv))!=0){
	fprintf(stderr,"Error in pairwise_comp()\n");
	exit(1);
      }

      pairwise_equivs[i][j] = pairwise_equivs[j][i] = pair_no_equiv;
      pairwise_rmsd[i][j] = pairwise_rmsd[j][i] = pair_rmsd;

      if((pair_no_equiv == (-99)) || (pair_rmsd == (-99.0)) || (pfte == (-99.0)))
	sdist[i][j] = sdist[j][i] = (-99.0);

      else
	sdist[i][j] = sdist[j][i] = calc_structural_dist_metric(i,j,cut_off,pair_rmsd,pfte);

   }
}

common_no_equiv = print_common_equiv(t_len,
				     ns,
				     all_equiv);

printf("\n\nNumber of common equivalences (all pairwise) = %d\n",common_no_equiv);

common_mat=imatrix(0,ns-1,0,t_len-1);

if((common_equiv(ns,all_equiv,common_mat))!=0){
  fprintf(stderr,"Error in common_equiv()\n");
  exit(1);
}

/*
   Pairwise rmsds over common (all pairwise) equivalences
*/

for(i=0;i<ns-1;i++){

   for(j=i+1;j<ns;j++){

      if((common_pairwise_comp(i,j,all_equiv,
			       common_mat,
			       t_len,ns,common_no_equiv,
			       &pair_rmsd))!=0){

	fprintf(stderr,"Error in common_pairwise_comp()\n");
	exit(1);
      }

      common_pairwise_rmsd[i][j] = common_pairwise_rmsd[j][i] = pair_rmsd;

   }
}


printf("\n\n******************************************************");
printf("*********************");
printf("\n************************ Multiple comparison statistics:");
printf(" ******************");
printf("\n********************************************************");
printf("*******************");

printf("\n\nPercentage identities:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i==j)fpid[family[0][i]-1][family[0][j]-1]=100.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.1f",fpid[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");


printf("\n\nDifferences:\n\n");
/*printf("\n\n100 * normalized differences:\n\n");*/
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i==j)fdif[family[0][i]-1][family[0][j]-1]=0.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.0f",fdif[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");


printf("\n\nLengths of alignment including overhangs, gaps:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(j>0 && j%10==0)printf("\n            ");
		if(i==j) printf("%6d",ren[family[0][i]-1]);
		else printf("%6d",nel_p[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");


printf("\n\nDistances from identities (D = -100.0 ln pids/100):\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i==j)fpid[family[0][i]-1][family[0][j]-1]=100.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.1f",-100.0 * log((fpid[family[0][i]-1][family[0][j]-1]/100.0)));
		}
	}
printf("\n");


printf("\n\nRaw alignment scores / 100 \n");
printf("[Overall (sum of off-diagonal elements): %8.1f]:\n\n",tscore/100.0);
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
	/*	if(i==j)fscore[family[0][i]-1][family[0][j]-1]=100.0;*/
		if(j>0 && j%10==0)printf("\n            ");
		printf(" %5.1f",rscore[family[0][i]-1][family[0][j]-1]/100.0);
		}
	}
printf("\n");


printf("\n\nNormalized alignment scores (NAS = 100.0 * Sreal/Smax):\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
	/*	if(i==j)fscore[family[0][i]-1][family[0][j]-1]=100.0;*/
		if(j>0 && j%10==0)printf("\n            ");
		printf(" %5.1f",fscore[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");


printf("\n\nDistances based on NAS scores [-100.0 * ln(NAS/100.0)]:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
	/*	if(i==j)fscore[family[0][i]-1][family[0][j]-1]=100.0;*/
		if(j>0 && j%10==0)printf("\n            ");
		printf(" %5.1f",
		(-100.0 * log(fscore[family[0][i]-1][family[0][j]-1]/100.0)));
		}
	}
printf("\n");

printf("\n\nPairwise equivalences in multiple superposition:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i == j)
		  pairwise_equivs[family[0][i]-1][family[0][j]-1] = ren[family[0][i]-1];

		if(j>0 && j%10==0)printf("\n            ");
		printf("%6d",pairwise_equivs[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");

printf("\n\nPairwise rmsds in multiple superposition:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i == j)
		  pairwise_rmsd[family[0][i]-1][family[0][j]-1] = 0.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.2f",pairwise_rmsd[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");

printf("\n\nPairwise structural distance metrics in multiple superposition:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i == j)
		  sdist[family[0][i]-1][family[0][j]-1] = 0.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.2f",sdist[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");

printf("\n\nNumber of common equivalences (all pairwise) = %d\n",common_no_equiv);
printf("\n\nPairwise rmsds over common (all pairwise) equivalences:\n\n");
printf("            ");
for(i=0;i<ns;i++)
	{
	if(i>0 && i%10==0)printf("\n            ");
	printf(" [%3d]",i+1);
	}
printf("\n");
for(i=0;i<ns;i++)
	{
	printf("\n%3d-%-8.8s",i+1,code[family[0][i]-1]);
	for(j=0;j<ns;j++)
		{
		if(i == j)
		  common_pairwise_rmsd[family[0][i]-1][family[0][j]-1] = 0.0;
		if(j>0 && j%10==0)printf("\n            ");
		printf("%6.2f",common_pairwise_rmsd[family[0][i]-1][family[0][j]-1]);
		}
	}
printf("\n");

/*
 *	exit
 */

printf("\n\n********************************************************");
printf("*******************");
printf("\n\n");

fclose(file);

free_ivector(all_equiv,0,t_len-1);
free_imatrix(common_mat,0,ns-1,0,t_len-1);

return 0;
}
